// Assume device already detectable to other devices
// sudo hciconfig hci0 piscan
// Assume MAC address of client and server known

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/rfcomm.h>
#include <pthread.h>
#include <stdbool.h>
 
// Initialize thread IDs
static pthread_t deviceDiscoveryId;
static pthread_mutex_t mutex;
 
// Initialize module variables
static bdaddr_t bluetoothAdapterID = {{0xAB, 0xCD, 0xEF, 0xGH, 0xIJ, 0xKL}}; // client MAC address
char userIDString[17] = "01:23:45:67:89:YZ";  // server MAC address
static bool userInRange = false;
static bool isDoneRunning = false;
 
// Initializes the Bluetooth module and starts thread. Must be run before other functions.
void BluetoothServer_init(void)
{
    pthread_create(&deviceDiscoveryId, NULL, deviceDiscoveryThread, NULL);
    return;
}
 
// Shuts downBlueTooth module and closes thread
void BluetoothServer_cleanup(void)
{
    isDoneRunning = true;
    pthread_join(deviceDiscoveryId, NULL);
    return;
}

// Checks if the user's device is within range
static bool BluetoothServer_isUserInRange(void)
{
    bool status = false;
    pthread_mutex_lock(&mutex);
    status = userInRange;
    pthread_mutex_unlock(&mutex);
    return status;
}
 
// This function looks up the Bluetooth device name from the known device address.
// If successful, function returns 0. If unsuccessful, function returns -1;
static int discoverDevice(void)
{
    int dev_id = 0;
    dev_id = hci_get_route(&bluetoothAdapterID);
 
    // Open a Bluetooth socket to connect BBG to the Bluetooth adapter
    int sock = 0;
    sock = hci_open_dev(dev_id);
 
    // Error checking
    if (dev_id < 0 || sock < 0)
    {
        perror("opening socket");
        exit(1);
    }
 
    char name[248] = {0};
 
    bdaddr_t userID;
    str2ba(userIDString, &userID);
 
    // Read name from device ID
    int rsp_code = hci_read_remote_name(sock, &userID, sizeof(name), name, 0);
 
    close(sock);
 
    return rsp_code;
}
 
// This thread continuously scans if the user is within range every 1 sec
static void *deviceDiscoveryThread(void *_)
{
    int scanPeriod = 1; // s
    while (!isDoneRunning)
    {
        int rsp_code = 0;
        rsp_code = discoverDevice();
 
        if (rsp_code < 0)
        {
            pthread_mutex_lock(&mutex);
            userInRange = false;
            pthread_mutex_unlock(&mutex);
        }
        else
        {
            pthread_mutex_lock(&mutex);
            userInRange = true;
            pthread_mutex_unlock(&mutex);
        }
        sleep(scanPeriod);
    }
    return NULL;
}

int main()
{
    // program runs for two minutes
    BluetoothServer_init();
    sleep(120);
    BluetoothServer_cleanup():
    return 0;
}